package gov.va.claims.attachments;

import gov.va.claims.attachments.database.AttachmentsSQLServerConnection;
import gov.va.claims.attachments.model.ClaimsAttachment;
import gov.va.claims.attachments.model.ClaimsAttachmentDAO;
import gov.va.claims.attachments.model.SQLServerClaimsAttachmentDAO;
import gov.va.claims.attachments.parser.Parser;
import gov.va.claims.attachments.parser.X275Simple;
import gov.va.claims.attachments.parser.X275SimpleParser;
import gov.va.claims.attachments.parser.helpers.Segment;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
// import java.nio.file.FileSystems;
// import java.nio.file.Path;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.List;
import java.util.Properties;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.GnuParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.OptionBuilder;
import org.apache.commons.cli.Options;
import org.apache.commons.io.FileUtils;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ch.qos.logback.classic.LoggerContext;
import ch.qos.logback.classic.joran.JoranConfigurator;
import ch.qos.logback.core.joran.spi.JoranException;
import ch.qos.logback.core.util.StatusPrinter;

/**
  * Sample File I/O classes - intial draft
// eclipse default text file input 
-f 275_edited.txt
-d C:\FileAttachmentsInput
-i 1
 
// command line binary file input 
	    -f 275_errata.pdf
		-d C:\FileAttachmentsInput
		-i 0
		-p1 PAPERprName
		-p2 PAPERprID
		-p3 PAPERpName
		-p4 PAPERpID
		-p5 PAPERcid
		-p6 20100720
		-p7 PAPERacn
	
   
 * @author DOMAINBLACKW
 * @since 8/4/2014
 */
public class AttachmentsMain {
	
	private static final boolean TEST_DB = true;
	private static final boolean TEST_CMD_LINE = true;
	
	static final Logger Log = LoggerFactory.getLogger(AttachmentsMain.class);
		
	private static int INPUT_TYPE = 1;			// 1= file input, 0 = paper
	
	static ClaimsAttachmentDAO claimsAttachmentDAO;

	// default constructor
	 public AttachmentsMain() { }
	
	 @SuppressWarnings({ "deprecation", "unused" })
	public static void main(String[] args) {
			 
		 System.setProperty("logback.xml", ".");

	
		 final Logger logMain = LoggerFactory.getLogger(AttachmentsMain.class);
		 
		 // used for debugging log output......
		 // LoggerContext lc = (LoggerContext) LoggerFactory.getILoggerFactory();
		 // StatusPrinter.print(lc);
		 
		 AttachmentsMain attachmentsMain = new AttachmentsMain();
		 Externals externals = new Externals();
		 Properties properties = new Properties();
		 HashMap cmdLineMap = new HashMap();
		 
		 ClaimsAttachment claimsAttachment = new ClaimsAttachment();
		 
		 AttachmentsSQLServerConnection con = null;
		 
		int inputType=0;
		 
		String sql_url="sql_url";						// jdbc:oracle:thin:@ISD-LTBLACKW7.vha.domain.ext
		String sql_port="sql_port";						// 1521
		String sql_database="sql_database";				//XE
		String sql_user="sql_user";						//system
		String sql_password="sql_password";				//password
			
		String url="";
		String port="";
		String database="";
		String password="";
		String user="";

		String sInputType = "";
		String sInputDirName = "";
		String sInputFileName = "";
// READ COMMAND LINE ARGS		
	        Options opts = getCliOptions();


	        try {
	            CommandLineParser parser = new GnuParser();
	            CommandLine line = null;
	            
		        line = parser.parse(opts, args);
		        
// if this is invalid, log and exit application	
// 1= file input, 0 = paper		        
          	   if (line.hasOption(Globals.INPUT_TYPE_OPT)) {
	        	   sInputType=line.getOptionValue(Globals.INPUT_TYPE_OPT);
	        	   
	        	   if(sInputType.equalsIgnoreCase("1")) {
	        		   inputType = 1;	// file input
	        		   // @SuppressWarnings({ "unused", "rawtypes" })
	        		   cmdLineMap = attachmentsMain.parseCmdLine(inputType, line, opts);   
	        		   
	        		   // PULL OUT DIR AND FILE NAMES
	        		   sInputDirName = (String) cmdLineMap.get("d");
	        		   sInputFileName = (String) cmdLineMap.get("f");
	        	   } else if(sInputType.equalsIgnoreCase("0")) {
	        		   inputType = 0; 	// paper input
	        		   cmdLineMap = attachmentsMain.parseCmdLine(inputType, line, opts);   
	        		   
	        		   // PULL OUT DIR AND FILE NAMES
	        		   sInputDirName = (String) cmdLineMap.get("d");
	        		   sInputFileName = (String) cmdLineMap.get("f");
	        		   
	        		   
	        	   } else {
	        		   logMain.error("Main() - invalid INPUT_TYPE_OPT cmd line option: " + sInputType + ", valid argments are 1 - file input or 0 paper input");
			    	   System.exit(1);
	        	   }
		       } else {
		    	   logMain.error("Main() - INPUT_TYPE_OPT cmd line option NOT FOUND - EXITING");
		    	   System.exit(1);
		       }
		    
	        }catch(org.apache.commons.cli.MissingOptionException moe) {
	        	logMain.error("main() - Missing command line options ERROR: " + moe.getMessage());
	        	moe.printStackTrace();
	        	System.exit(1);
	        }catch(Exception ex) {
				ex.printStackTrace();
				// logMain.error("message");
				logMain.error("main() - error reading command line args - exiting main()");
				logMain.debug("main() - error reading command line args - exiting main()");
				System.exit(1);
	        }
		          	   

			
// if ANYTHING fails here, exit..........
if(TEST_DB) {	
	
	
	// read external props.
				try {
					externals.setProperties();
				} catch (Exception e1) {
					e1.printStackTrace();
				}
				properties = externals.getConfigProps();
				
				Enumeration e = properties.keys();
	
			try {
			 while (e.hasMoreElements()) {
				 
		          String key = (String) e.nextElement();
		          
// GET SQL SERVER ARGS
		          if(key.equalsIgnoreCase(sql_url)) {
		        	  url=(String) properties.getProperty(key);
		        	  if( (url=="") || (url.length()==0 ) || (url==null) ) {
		        		  logMain.error("main() empty sql_url in config.properties file - exiting");
		        		  return;
		        	  }
		          } 
		          
		          if(key.equalsIgnoreCase(sql_port)) {
		        	  port=(String) properties.getProperty(key);
		        	  if( (port=="") || (port.length()==0 ) || (port==null) ) {
		        		  logMain.error("main() empty sql_port in config.properties file - exiting");
		        		  return;
		        	  }
		          } 
		          
		          if(key.equalsIgnoreCase(sql_database)) {
		        	  database=(String) properties.getProperty(key);
		        	  if( (database=="") || (database.length()==0 ) || (database==null) ) {
		        		  logMain.error("main() empty sql_database in config.properties file - exiting");
		        		  return;
		        	  }
		          }
		          
		          if(key.equalsIgnoreCase(sql_user)) {
		        	  user=(String) properties.getProperty(key);
		        	  if( (user=="") || (user.length()==0 ) || (user==null) ) {
		        		  logMain.error("main() empty sql_user in config.properties file - exiting");
		        		  return;
		        	  }
		          }
		          
		          if(key.equalsIgnoreCase(sql_password)) {
		        	  password=(String) properties.getProperty(key);
		        	  if( (password=="") || (password.length()==0 ) || (password==null) ) {
		        		  logMain.error("main() empty sql_password in config.properties file - exiting");
		        		  return;
		        	  }
		          }
			 }	// end while reading properties file     
		          
			} catch(Exception ex) {
				ex.printStackTrace();
				logMain.error("main() - error reading config.properties file - exiting main()");
				return;
			}
			// end db connectiion

}	// end DB CONNECTION test
// END READING OF DB CONFIG ITEMS...........

	
			
			try {

				// read input data file here.................
				// jdk 7 .nio only
				// Path path = FileSystems.getDefault().getPath(sInputDirName, sInputFileName);
				// File f1 = path.toFile();
				
				// jdk 6
				String sFilePath=sInputDirName+ "\\" + sInputFileName;
// System.out.println("AttachmentsMain() - FILE PATH IS: " + sFilePath);

logMain.debug("LOG MAIN DEBUG OUTPUT TEST - file and directory test is: " + sFilePath);
				
				
				File f1 = new File(sFilePath);
				
				// file input type 
				// if the file is Paper i.e. scanned, I am assuming it is a binary file. It will be read as binary and the 
				// contents will be written as varchar into DB as required (probably incorrectly) in the SDD.....
				if(inputType == 1) {
					// do this first - parse fully-formed object
					Parser parserSimple = new X275SimpleParser();
					
					X275Simple x275s = (X275Simple) parserSimple.parse(f1);
					claimsAttachment = attachmentsMain.parse275(x275s);
					
				// paper input type	i.e. a SCANNED binary document
				} else if (inputType == 0) {
					// read entire file into one field..........how to do this best?
					// String sFullFile = x275s.toString();
					
					// get a path to the file and read the binary contents regardless of file type since we won't know what it it....
					
					// we already have file location via F1
					// get a handle to the file and get it's size
					long fileSize = f1.length();
					// can it be read?
					boolean bCanRead = f1.canRead();
					// full text contents to be written after binary conversion because of datatype requirement in the DB
					String sFullFile = "";
					byte[] binaryIn;
					
					if(bCanRead) {
						// open a stream.....
						boolean isBinary = false;
						
						isBinary = isBinaryFile(f1);
						
						if(!isBinary) {
							Parser parserSimple = new X275SimpleParser();
							X275Simple x275s = (X275Simple) parserSimple.parse(f1);
							sFullFile = x275s.toString();
						} else {
							// pull out binary value and convert to text somehow and write to database
// System.out.println("main() - Binary file size is: " + f1.length());

							byte[] bytes = FileUtils.readFileToByteArray(f1);	// throws IOException
// System.out.println("main() - Binary byte array is: " + bytes.toString());

//							System.out.println("Binary byte count is: " + bytes.length);
							sFullFile = new String(bytes); 
							
// System.out.println("\nmain() - String value of binary file is: " + sFullFile);							
							
// System.out.println("\nBinary-to-String sFullFile size is: " + sFullFile.length());
						}
					}
					
				 String sPN = (String) cmdLineMap.get("p1");			// provider name
				 String sPID = (String) cmdLineMap.get("p2");			// provider ID
				 String sPatN = (String) cmdLineMap.get("p3");			// patient name
				 String sPatID = (String) cmdLineMap.get("p4");			// patient ID - NULL 
				 String sCID = (String) cmdLineMap.get("p5");			// claim ID - NULL
				 String sCSD = (String) cmdLineMap.get("p6");			// claim service date
				 String sACN = (String) cmdLineMap.get("p7");			// full file
				 
				 claimsAttachment = new ClaimsAttachment(sPN, sPID, sPatN, sPatID, sCID, sCSD, sACN, sFullFile);
				 
				}	// end of input type check
				
				con = new AttachmentsSQLServerConnection(url, database, user, password, port);
				 
				
				if(con!=null) {
					SQLServerClaimsAttachmentDAO sqlDAO = new SQLServerClaimsAttachmentDAO();
					sqlDAO.insertClaimsAttachment(con, claimsAttachment);
				}	
			} catch (Exception e1) {
				logMain.error(e1.getMessage());
				e1.getMessage();
				e1.printStackTrace();
			} finally {
				con.closeConnection();
				con = null;
			}
}	// end main()
	 

		/**
		 * TO DO:
		  * Parses x275 object and pulls off relevant fields
		 * @author DOMAINBLACKW
		 * @since 9/17/2014 
		 * @param x275s
		 * @return ClaimsAttachment
		 */
		private ClaimsAttachment parse275(X275Simple x275s) {
			
			boolean DEBUG = true;
			
			// attachment fields
			// PROVIDER FIELDS
			String sProviderLastName = "";
			String sProviderFirstName = "";
			String sProviderMidName = "";
			String sProviderFullName = "";
			String sProviderID = "";
			
			String sPatientLastName = "";
			String sPatientFirstName = "";
			String sPatientMidName = "";
			String sPatientFullName = "";
			String sPatientID = "";
			
			// CLAIM FIELDS
			String sClaimIdentifier = "";
			String sClaimServiceDate = "";
			
			// ATTACHMENT CONTROL NUMBER
			String sAttachmentControlNumber = "";
			
			// BDS
			String sBDS = "";
			
			// Full File - may need to be a BLOB....
			String sFullFile = "";
			
			ClaimsAttachment claimsAttachment = null;
				
			/**
			 * NM1, REF, DTP, TRN, BDS - segment names
			 * 
			 */
			sFullFile = x275s.toString();
			
if(DEBUG) {			
 System.out.println("parse275() - fullString from 275 is:" + sFullFile);			
}

			List<Segment> allSegments = x275s.getSegments();

			// loop through segs ONCE and identifiy IDs.....
			String sId = "";
			for (Segment s : allSegments) {
				
				 sId = s.getElement(0);
				 
				 if(sId.equalsIgnoreCase("NM1") ) {
					 // System.out.println("NM1 found");
					 
					 // get Provider Info.
					 // //PROVIDER NAME and PROVIDER IDENTIFICATION NUMBER	
					 if(s.getElement(1).equalsIgnoreCase("1P")) {
							
							sProviderLastName = s.getElement(3);
							sProviderFirstName = s.getElement(4);
							sProviderMidName = s.getElement(5);
							sProviderID = s.getElement(9);
							
							sProviderID = sProviderID.replaceAll("~", "");
							sProviderFullName = sProviderLastName + ", " + sProviderFirstName + " " + sProviderMidName;
	
if(DEBUG) {							
		System.out.println("parse275() - Provider Full Mame is: " + sProviderFullName);
		System.out.println("parse275() - ProviderID is: " + sProviderID);
}							
							// break;
						}	// end IP
					 
					 	//PATIENT NAME
						//NM1 - PATIENT NAME
						//pg. 52, draft doc.				
						//QC Patient element(1) - Required				
						//NM1✽QC✽1✽SMITH✽JOHN✽Q✽✽IV✽MI✽99887777~
						//elements(2.3.4) concatenated	- element 2 Required	
					 if(s.getElement(1).equalsIgnoreCase("QC")) {
							
							sPatientLastName = s.getElement(3);
							sPatientFirstName = s.getElement(4);
							sPatientMidName = s.getElement(5);
							
							sPatientID = s.getElement(9);	
							sPatientID = sPatientID.replaceAll("~", "");
							
							sPatientFullName = sPatientLastName + ", " + sPatientFirstName + " " + sPatientMidName;
if(DEBUG) {							
			System.out.println("parse275() - Patient Full Name is: " + sPatientFullName);
			System.out.println("parse275() - Patient ID is: " + sPatientID);
}							
							// break;
						}	// end QC
				 }	// end NM1
				 
				 
				 
// CLAIM ID - VERIFY THAT THIS IS RIGHT
				 if(sId.equalsIgnoreCase("REF") ) {
					 
					 
					 if(s.getElement(1).equalsIgnoreCase("D9")) {
						 String temp = s.getElement(2);
						
						 // this is pulling in the '~' at the end of the String - trim it off.....
						 temp = temp.replaceAll("~", "");
						 
if(DEBUG) {						 						 
		 System.out.println("parse275() - Claim Identifier number is: " + temp);
}		 

						sClaimIdentifier = temp;
						//	break;
						}
				 }	// end REF

// CLAIM SERVICE DATE - VERIFY THAT THIS IS CORRECT				 
				 if(sId.equalsIgnoreCase("DTP") ) {
					 
					 
					 if(s.getElement(1).equalsIgnoreCase("434")) {		// qualifier is 434
							String temp = s.getElement(3);
							
							// this is pulling in the '~' at the end of the String - trim it off.....
							sClaimServiceDate = temp.replaceAll("~", "");
							
if(DEBUG) {							
		System.out.println("parse275() - Claim Service Date is: " + sClaimServiceDate);
}		
							
							
							// break;
					}
				 }	// end DTP
				 
				 
				//ATTACHMENT CONTROL NUMBER 
				//TRN*2*1234567~	
				 if(sId.equalsIgnoreCase("TRN") ) {
					
					 
					 if(s.getElement(1).equalsIgnoreCase("2")) {		// qualifier is 2
							String temp = s.getElement(2);
							
							// this is pulling in the '~' at the end of the String - trim it off.....
							sAttachmentControlNumber = temp.replaceAll("~", "");
if(DEBUG) {							
		System.out.println("parse275() - Attachment Control Number is: " + sAttachmentControlNumber);
}
							
							// break;
					}
				 }	// end TRN
				 
				 if(sId.equalsIgnoreCase("BDS") ) {
					 String temp = s.getElement(3);
						
					// this is pulling in the '~' at the end of the String - trim it off.....
					 sBDS = temp.replaceAll("~", "");
					 
if(DEBUG) {					 
					 System.out.println("parse275() - Binary Data is: " + sBDS);
}
					 
					 
				 }	// end BDS
				 
			}	// end Segment check For
			
			
			
			claimsAttachment = new ClaimsAttachment(sProviderFullName, sProviderID, sPatientFullName, sPatientID,
					sClaimIdentifier, sClaimServiceDate, sAttachmentControlNumber,
					sBDS, sFullFile);	
			
			
			// return claimsAttachment
			return claimsAttachment;
			
		}	// end parse275()	
	
	 /**
	  * This method instantiates the correct DAOFactory based on args from the config.properties file
	  * @author DOMAINBLACKW
	  * @since 8/25/2014
	  * @param url
	  * @param database
	  * @param user
	  * @param password
	  * @param port
	  */
	
	 /**
	     * Helper method to populate command line options objects.
	     *
	     * @return the <code>Options</code> object containing the set of command line
	     *         option.
	     */
	    @SuppressWarnings("static-access")
	    static Options getCliOptions()  {
	        Options options = new Options();
	        
	        
	        Option help = OptionBuilder.withLongOpt("help")
                    .withDescription("Print this message.")
                    .create(Globals.HELP_OPT);
	 	        
	        Option inputFile = OptionBuilder.withArgName("input file").isRequired()
                    .hasArg().withLongOpt("inputFileName")
                    .withDescription("The Input File Name.")
                    .create(Globals.INPUT_FILE_NAME_OPT);
	        
	        
	        Option inputDirectory = OptionBuilder.withArgName("input directory").isRequired()
                    .hasArg().withLongOpt("inputDirectory")
                    .withDescription("The Input Directory.")
                    .create(Globals.INPUT_DIRECTORY_OPT);

// CHANGE TO INPUT_OPT, 'i'........	        
	        Option inputType = OptionBuilder.withArgName("input type").isRequired()
                    .hasArg().withLongOpt("inputType")
                    .withDescription("The Input Type.")
                    .create(Globals.INPUT_TYPE_OPT);
	          
// START OF PAPER INPUT TYPE COMMAND LINE ARGS...
	     // provider ID	        
	        Option providerID = OptionBuilder.withArgName("provider id")
                    .hasArg().withLongOpt("providerID")
                    .withDescription("The Provider ID.")
                    .create(Globals.PROVIDER_ID_OPT);
	        
	        
// not required, multiple words.......        
	        Option providerName = OptionBuilder.withArgName("provider name")
                    .hasArg().withLongOpt("provider Name")
                    .withDescription("The Provider Name.")
                    .hasArgs()
                    .create(Globals.PROVIDER_NAME_OPT);
	        
	        Option patientName = OptionBuilder.withArgName("patient name")
                    .hasArg().withLongOpt("patient Name")
                    .withDescription("The Patient Name.")
                    .hasArgs()
                    .create(Globals.PATIENT_NAME_OPT);
	        
	        Option patientID = OptionBuilder.withArgName("patient identifier")
                    .hasArg().withLongOpt("patient identifier")
                    .withDescription("The Patient Identifier.")
                    .hasArgs()
                    .create(Globals.PATIENT_ID_OPT);
	        
	        Option claimIdentifier = OptionBuilder.withArgName("claim identifier")
                    .hasArg().withLongOpt("claim indentifier")
                    .withDescription("The Claim Identifier.")
                    .hasArgs()
                    .create(Globals.CLAIM_IDENTIFIER_OPT);
	        
	        Option claimServiceDate = OptionBuilder.withArgName("claim service date")
                    .hasArg().withLongOpt("claim service date")
                    .withDescription("The Claim Service Date.")
                    .hasArgs()
                    .create(Globals.CLAIM_SERVICE_DATE_OPT);
	        
	        Option attachmentControlNumber = OptionBuilder.withArgName("attachment control number")
                    .hasArg().withLongOpt("attachment control number")
                    .withDescription("The Attachment Control Number.")
                    .hasArgs()
                    .create(Globals.ATTACHMENT_CONTROL_NUMBER_OPT);
	        
	        options.addOption(help);
	        options.addOption(inputFile);
	        options.addOption(inputDirectory);
	        options.addOption(inputType);
	        
	        // CONDITIONAL ARGS.....
	        options.addOption(providerName);
	        options.addOption(providerID);
	        options.addOption(patientName);
	        options.addOption(patientID);
	        options.addOption(claimIdentifier);
	        options.addOption(claimServiceDate);
	        options.addOption(attachmentControlNumber);
	    
	
	        return options;
	    }	// end getCliOptions()
	    
	    
	    /**
	     * This method parses the read command line args and returns them to main() - rewritten 
	     * @author DOMAINBLACKW
	     * @since 9/23/2014
	     * @param iInputType
	     * @param line
	     * @param opts
	     * @return HashMap
	     */
	    
	    
	    @SuppressWarnings("unchecked")
		private HashMap parseCmdLine(int iInputType, CommandLine line, Options opts) {
			
	/** FULL LIST OF COMMAND LINE OPTIONS 
	 TO DO: test with only first 3 options with i=1 (file input) 
	/**
	    -f 275_errata.pdf
		-d C:\FileAttachmentsInput
		-i 0
		-p1 PAPERprName
		-p2 PAPERprID
		-p3 PAPERpName
		-p4 PAPERpID
		-p5 PAPERcid
		-p6 20100720
		-p7 PAPERacn
	 */
	
	    	boolean DEBUG = true;
	    	
			HashMap map = new HashMap();
			
			
			// command line options
			String INPUT_FILE_NAME_OPT = "f";
			String INPUT_DIRECTORY_OPT = "d";
		 	String HELP_OPT = "h";
		 	String INPUT_TYPE_OPT = "i";	// 0 = paper, 1 = file - DEFAULT ECLIPSE VAL. = 1, File
		 	String PROVIDER_NAME_OPT = "p1";
		 	String PROVIDER_ID_OPT = "p2";
		 	String PATIENT_NAME_OPT = "p3";
		 	String PATIENT_ID_OPT = "p4";
		 	String CLAIM_IDENTIFIER_OPT = "p5";
		 	String CLAIM_SERVICE_DATE_OPT = "p6";
		 	String ATTACHMENT_CONTROL_NUMBER_OPT = "p7";
		 	
		  	
		 	// passed in as cmd line args
		 	String sInputFileName = "";
		 	String sInputDirName = "";
		 	String sInputType = "";
		 	String sProviderName = "";
		 	String sProviderID = "";
		 	String sPatientName = "";
		 	String sPatientID = "";
		 	String sClaimIdentifier = "";
		 	String sClaimServiceDate = "";
		 	String sAttachmentControlNumber = "";
		 	
		 	 if (line.hasOption(HELP_OPT)) {
	        	   // Log.info("HELP OPTION FOUND");	
	        	   map.put(HELP_OPT, "HELP");
	               printUsage(opts);
	               System.exit(1);
	           }  
	           
	           if (line.hasOption(INPUT_FILE_NAME_OPT)) {
	        	   sInputFileName=line.getOptionValue(INPUT_FILE_NAME_OPT);
	        	   map.put(INPUT_FILE_NAME_OPT, sInputFileName);

if(DEBUG) {	        	   
	        	   System.out.println("parseCmdLine() - INPUT_FILE_NAME_OPT FOUND: " + sInputFileName);
} 
	           }	// end if 
	           
	           
	           if (line.hasOption(INPUT_DIRECTORY_OPT)) {
		        	sInputDirName=line.getOptionValue(INPUT_DIRECTORY_OPT);
		        	map.put(INPUT_DIRECTORY_OPT, sInputDirName);

if(DEBUG) {			        	
		        	System.out.println("INPUT_DIRECTORY_OPT FOUND: " + sInputDirName);
}	
		        	
		       }	// end if
	           
	           
	           // // 1= file input, 0 = paper/binary scan	
	           if(iInputType==0) {
	        	   
			           	if(line.hasOption(PROVIDER_NAME_OPT)) {
		         		   
			         		 String[] pArgs = line.getOptionValues(PROVIDER_NAME_OPT);
			         		 
				             String temp = "";
				             for (String p : pArgs) {
				            	 temp += p + " ";
				             }	// end for
				             
				             sProviderName = temp;
				             map.put(PROVIDER_NAME_OPT, sProviderName);
				             
if(DEBUG) {				             
			System.out.println("parseCmdLine() - provider name is: " + sProviderName);
}	
		
			         	   }	// end provider name check
			            
			           	
			         	   if(line.hasOption(PROVIDER_ID_OPT)) {
			         		   
			         		 String[] pArgs = line.getOptionValues(PROVIDER_ID_OPT);
			         		 
				             String temp = "";
				             for (String p : pArgs) {
				            	 temp += p + " ";
				             }	// end for
			         		   
			         		   sProviderID=temp;
			         		   map.put(PROVIDER_ID_OPT, sProviderID);
			         		   
if(DEBUG) {			         		   
			System.out.println("parseCmdLine() - Provider ID is :" + sProviderID);
}
			         		   
			         		   
			         	   }	// end provider id option
		     	        
			         	  if(line.hasOption(PATIENT_NAME_OPT)) {
			         		   
				          		 String[] pArgs = line.getOptionValues(PATIENT_NAME_OPT);
				          		 
					             String temp = "";
					             for (String p : pArgs) {
					            	 temp += p + " ";
					             }
					             
					             sPatientName = temp;
					             map.put(PATIENT_NAME_OPT, sPatientName);
					             
if(DEBUG) {						             
			System.out.println("parseCmdLine() - patient name is: "+ sPatientName);	
}	
				           }	// end patient name check  
			         	  
			         	  
			         	  
			         	  if(line.hasOption(PATIENT_ID_OPT)) {
			         		   
				          		 String[] pArgs = line.getOptionValues(PATIENT_ID_OPT);
				          		 
					             String temp = "";
					             for (String p : pArgs) {
					            	 temp += p + " ";
					             }
					             
					             sPatientID = temp;
					             map.put(PATIENT_ID_OPT, sPatientID);
					             
if(DEBUG) {						             
			System.out.println("parseCmdLine() - patient ID is: "+ sPatientID);	
}	
				           }	// end patient id 
			         	  			         	  
			         	  
		         	   
				         	 if(line.hasOption(CLAIM_SERVICE_DATE_OPT)) {
				         		   
				         		 String[] pArgs = line.getOptionValues(CLAIM_SERVICE_DATE_OPT);
				         		 
					             String temp = "";
					             for (String p : pArgs) {
					            	 temp += p + " ";
					             }
					             sClaimServiceDate = temp;
					             map.put(CLAIM_SERVICE_DATE_OPT, sClaimServiceDate);
					             
if(DEBUG) {						             
				System.out.println("parseCmdLine() - claim service date is: "+ sClaimServiceDate);				             
}
				     	     
				         	 } // end if
		         	 
				         	 if(line.hasOption(CLAIM_IDENTIFIER_OPT)) {
			         		   
				          		 String[] pArgs2 = line.getOptionValues(CLAIM_IDENTIFIER_OPT);
				          		 
					             String temp2 = "";
					             for (String p : pArgs2) {
					            	 temp2 += p + " ";
					             }
					             sClaimIdentifier = temp2;
					             map.put(CLAIM_IDENTIFIER_OPT, sClaimIdentifier);
					             
if(DEBUG) {						             
	System.out.println("parseCmdLine() - claim identifier is: "+ sClaimIdentifier);	
}
			
					             
				         	 }	// end if
		         	   
		           	   
			         	  if(line.hasOption(ATTACHMENT_CONTROL_NUMBER_OPT)) {
			          		   
			          		 String[] pArgs3 = line.getOptionValues(ATTACHMENT_CONTROL_NUMBER_OPT);
			          		 
			 	             String temp3 = "";
			 	             for (String p : pArgs3) {
			 	            	 temp3 += p + " ";
			 	             }	// end for
			 	             
			 	             sAttachmentControlNumber = temp3;
			 	             
if(DEBUG) {						             
	System.out.println("parseCmdLine() - ACN: "+ sAttachmentControlNumber);	
}			 	             
			 	             
			 	             
			 	             
			 	             map.put(ATTACHMENT_CONTROL_NUMBER_OPT, sAttachmentControlNumber);
			         	  }	// end if 
			         	  
	    	}	// end input type check
	    
	    
			return map;
			
		}	// end parseCmdLine()
	    
	    
	    /**
	     *  Guess whether given file is binary. Just checks for anything under 0x09.
	     */
	    public static boolean isBinaryFile( File f ) throws FileNotFoundException, IOException {
	        FileInputStream in = new FileInputStream(f);
	        int size = in.available();
	        
	        if(size > 1024) size = 1024;
	        byte[] data = new byte[size];
	        in.read(data);
	        in.close();

	        int ascii = 0;
	        int other = 0;

	        for( int i = 0; i < data.length; i++ ) {
	            byte b = data[i];
	            if( b < 0x09 ) return true;

	            if( b == 0x09 || b == 0x0A || b == 0x0C || b == 0x0D ) ascii++;
	            else if( (b >= 0x20)  &&  b <= 0x7E ) ascii++;
	            else other++;
	        }

	        if( other == 0 ) return false;

	        return 100 * other / (ascii + other) > 95;
	    }
	    
	    
	    
	    
	    static final void printUsage(Options opts) {
	        final int helpwidth = 80;
	        HelpFormatter fmtr = new HelpFormatter();
	        System.out.println();
	        fmtr.printHelp(helpwidth, "File Attachments", "", opts, "", true);
	    }
	 
		public static ClaimsAttachmentDAO getClaimsAttachmentDAO() {
			return claimsAttachmentDAO;
		}

		public static void setClaimsAttachmentDAO(
				ClaimsAttachmentDAO claimsAttachmentDAO) {
			AttachmentsMain.claimsAttachmentDAO = claimsAttachmentDAO;
		}

			 

}	// end class


