
package gov.va.claims.attachments.parser;

import gov.va.claims.attachments.parser.helpers.Cf;
import gov.va.claims.attachments.parser.helpers.Context;
import gov.va.claims.attachments.parser.helpers.EDI;
import gov.va.claims.attachments.parser.helpers.FormatException;
import gov.va.claims.attachments.parser.helpers.Loop;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.util.Scanner;
import java.util.regex.Pattern;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * The class represents methods used to translate a X12 transaction represented
 * as a file or string into an X12 object.
 * 

 */
public class AttachmentsParser implements Parser {
	
	static final Logger Log = LoggerFactory.getLogger( AttachmentsParser.class);
	
	boolean DEBUG = false;
	

	private static final int SIZE = 106;
	public static final int POS_SEGMENT = 105;				// set to 105 - lenght of ISA in demo data - changed to  for test this needs to be large enough to read an entire line - find the longest
															// line in the spec and set to that +1 and experiment
	public static final int POS_ELEMENT = 3;
	public static final int POS_COMPOSITE_ELEMENT = 104;

	private Cf x275Cf;
	private Cf cfMarker;
	private Loop loopMarker;

	public AttachmentsParser(Cf cf) {

if(DEBUG) {		
// System.out.println("X12Parser constr() - Cf passed is: " + cf.toString());	
}
		this.x275Cf = cf;
	}

	/**
	 * The method takes a X12 file and converts it into a X2 object. The X12
	 * class has methods to convert it into XML format as well as methods to
	 * modify the contents.
	 * 
	 * @param fileName
	 *            a X12 file
	 * @return the X12 object
	 * @throws FormatException
	 * @throws IOException
	 */
	@Override
	public EDI parse(File fileName) throws FormatException, IOException {
		
		long fileSize = fileName.length();
		
if(DEBUG) {
	System.out.println("X12Parser.parse() file size of input file is: " + fileSize);
}
		// final char[] buffer = new char[SIZE];
		
		final char[] buffer = new char[(int) fileSize];
		
		FileReader fr = new FileReader(fileName);
		int count = fr.read(buffer);
		
		
		fr.close();
		
		// LEAVE THIS OUT UNTIL A PROPER INPUT FILE IS LOCATED
		/**
		if (count != SIZE) {
			throw new FormatException();
		}
		**/
		
		
		Context context = new Context();
		
		// what is the purpose of these? these constants are wrong.........
		context.setSegmentSeparator(buffer[POS_SEGMENT]);
		context.setElementSeparator(buffer[POS_ELEMENT]);
		context.setCompositeElementSeparator(buffer[POS_COMPOSITE_ELEMENT]);

		Scanner scanner = new Scanner(fileName);
			
		X275 x275 = scanSource(scanner, context);
		scanner.close();
		
		
		return x275;
	}

	/**
	 * private helper method
	 * @param scanner
	 * @param context
	 * @return
	 */
	private X275 scanSource(Scanner scanner, Context context) {
		
		
		Character segmentSeparator = context.getSegmentSeparator();

if(DEBUG) {		
  System.out.println("AttachmentsParser.scanSource() - segmentSeparator is: " + segmentSeparator);
}
		
		String quotedSegmentSeparator = Pattern.quote(segmentSeparator.toString());
		// String quotedSegmentSeparator = Pattern.    // .quote(segmentSeparator.toString());
		
		/**
		 * with a newline character or that end with a carriage return and a newline character. You can do that using the regular expression "\r\n|\n".
		 */
		
		scanner.useDelimiter(quotedSegmentSeparator + "\r\n|" + quotedSegmentSeparator + "\n|" + quotedSegmentSeparator);

		cfMarker = x275Cf;
		X275 x275 = new X275(context);
		loopMarker = x275;
		Loop loop = x275;
		
		int nm1Count = 0;

		
// this doesn't move line by line, but by fixed length into the line - NOT WHAT I WANT.		
		while (scanner.hasNext()) {
			String lineElement = scanner.next();

// make sure ALL lines are parsed first 			
if(DEBUG) {
	System.out.println("\nAttachmentsParser.scanSource() - length of line element parsed is: " + lineElement.length());
}


			


			String[] tokens = lineElement.split("\\" + context.getElementSeparator());
			
			for(int i = 0; i < tokens.length; i++) {
				String out = tokens[i];
				
				if(DEBUG) {
					System.out.println("Token is: " + out);	
				}
				
				
				// capture first element in the line 
				if(i==0) {
					
// System.out.println("Length of read is: " + out.length());					
					if(out.length() > 0) {
						// System.out.println("FIRST LINE ELEMENT IS: " + out);
					}
					
				}
			
			}
			
			
			
if(DEBUG) {	
 // System.out.println("cfMarker is: " + cfMarker);	
 // System.out.println("X12Parser.scanSource() - line element tokens are: " + tokens.)	;
}
			
			if (doesChildLoopMatch(cfMarker, tokens)) {
				loop = loop.addChild(cfMarker.getName());

				if(DEBUG) {
					System.out.println("AttachmentsParser.scanSource() LOOPING - CHILD LOOP is: " + loop.toString());
				}				
				loop.addSegment(lineElement);
				
				
				if(DEBUG) {
					System.out.println("AttachmentsParser.scanSource() APPENDED CHILD LOOP is: " + loop.toString());
				}				

				
				
			} else if (doesParentLoopMatch(cfMarker, tokens, loop)) {
				loop = loopMarker.addChild(cfMarker.getName());
				
				if(DEBUG) {
					 System.out.println("AttachmentsParser.scanSource() LOOPING - PARENT LOOP is: " + loop.toString());
				}				
				loop.addSegment(lineElement);
				
				
				if(DEBUG) {
					System.out.println("AttachmentsParser.scanSource() APPENDED PARENT LOOP is: " + loop.toString());
				}				
				
				
			} else {
				loop.addSegment(lineElement);
			}
		}
		
		
		if(DEBUG) {
			
			System.out.println("AttachmentsParser.scanSource() - end - returning x12 value of: " + x275 + "\n");
			
		}
		
	
		return x275;
	}	// end scanSource()
	
	/**
	 * The method takes a InputStream and converts it into a X12 object. The X12
	 * class has methods to convert it into XML format as well as methods to
	 * modify the contents.
	 * 
	 * @param source
	 *            InputStream
	 * @return the X12 object
	 * @throws FormatException
	 * @throws IOException
	 */
	@Override
	public EDI parse(InputStream source) throws FormatException, IOException {
		StringBuilder strBuffer = new StringBuilder();
		char[] cbuf = new char[1024];
		int length = -1;

		Reader reader = new BufferedReader(new InputStreamReader(source));

		while ((length = reader.read(cbuf)) != -1) {
			strBuffer.append(cbuf, 0, length);
		}

		String strSource = strBuffer.toString();
		return parse(strSource);
	}
		
	/**
	 * The method takes a X12 string and converts it into a X2 object. The X12
	 * class has methods to convert it into XML format as well as methods to
	 * modify the contents.
	 * 
	 * @param source
	 *            String
	 * @return the X12 object
	 * @throws FormatException
	 * @throws IOException
	 */
	@Override
	public EDI parse(String source) throws FormatException {
		
		if (source.length() < SIZE) {
			throw new FormatException();
		}
		Context context = new Context();
		context.setSegmentSeparator(source.charAt(POS_SEGMENT));
		context.setElementSeparator(source.charAt(POS_ELEMENT));
		context.setCompositeElementSeparator(source.charAt(POS_COMPOSITE_ELEMENT));

		Scanner scanner = new Scanner(source);
		X275 x275 = scanSource(scanner, context);
		scanner.close();
		return x275;
	}
 
	/**
	 * Checks if the segment (or line read) matches to current loop
	 * 
	 * @param cf
	 *            Cf
	 * @param tokens
	 *            String[] represents the segment broken into elements
	 * @return boolean
	 */
	private boolean doesLoopMatch(Cf cf, String[] tokens) {
		
// IF INPUT IS ** this fails index	
/**
 * java.lang.ArrayIndexOutOfBoundsException: 0		
 */
		
	try {	
		if (cf.getSegment().equals(tokens[0])) {
			
			if (null == cf.getSegmentQualPos()) {
				return true;
			} else {
				for (String qual : cf.getSegmentQuals()) {
					if (qual.equals(tokens[cf.getSegmentQualPos()])) {
						return true;
					}
				}
			}
		}
	}catch(java.lang.ArrayIndexOutOfBoundsException e) {
		System.out.println("doesLoopMatch() ArrayIndexOutOfBounds error: " + e.getMessage());
		
	}
		
		return false;
	}

	/**
	 * Checks if the segment (or line read) matches to any of the child loops
	 * configuration.
	 * 
	 * @param cf
	 *            Cf
	 * @param tokens
	 *            String[] represents the segment broken into elements
	 * @return boolean
	 */
	boolean doesChildLoopMatch(Cf parent, String[] tokens) {
		
		for (Cf cf : parent.childList()) {
			
			if (doesLoopMatch(cf, tokens)) {
				cfMarker = cf;
				return true;
			}
		}
		
		return false;
	}

	/**
	 * Checks if the segment (or line read) matches the parent loop
	 * configuration.
	 * 
	 * @param cf
	 *            Cf
	 * @param tokens
	 *            String[] represents the segment broken into elements
	 * @param loop
	 *            Loop            
	 * @return boolean
	 */
	private boolean doesParentLoopMatch(Cf child, String[] tokens, Loop loop) {
		Cf parent = child.getParent();
		if (parent == null)
			return false;
		
		loopMarker = loop.getParent();
		for (Cf cf : parent.childList()) {
			if (doesLoopMatch(cf, tokens)) {
				cfMarker = cf;
				return true;
			}
		}
		
		if (doesParentLoopMatch(parent, tokens, loopMarker))
			return true;
		
		return false;
	}
}
